'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pullTask = exports.commitTask = exports.cleanTask = exports.installTask = exports.removeTask = exports.addTask = exports._installApp = exports._cleanApp = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _bluebird = require('bluebird');

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _console = require('../../utils/console');

var _exec = require('../../utils/exec');

var _exec2 = _interopRequireDefault(_exec);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cachedConfig = null;

var cachedApps = null;

var appsPath = _path2.default.join('apps');

var configPath = _path2.default.resolve('maha.js');

var _readConfig = function _readConfig() {

  if (cachedConfig) return cachedConfig;

  cachedConfig = require(configPath);

  return cachedConfig;
};

var _writeConfig = function _writeConfig(config) {

  var data = 'module.exports = ' + JSON.stringify(config, null, 2);

  _fs2.default.writeFileSync(configPath, data, 'utf8');
};

var _getApps = function _getApps() {

  if (cachedApps) return cachedApps;

  var config = _readConfig();

  cachedApps = config.apps;

  return cachedApps;
};

var _validateApp = function _validateApp(name) {

  var apps = Object.keys(_getApps());

  if (!_lodash2.default.includes(apps, name)) throw new Error('app ' + name + ' is not installed');
};

var executeForOneOrAll = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(handler, name, arg) {
    var argument, apps;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            argument = typeof arg === 'string' ? arg : null;

            if (!(typeof name === 'string' && name !== 'all')) {
              _context2.next = 6;
              break;
            }

            _validateApp(name);

            _context2.next = 5;
            return handler(name, argument);

          case 5:
            return _context2.abrupt('return', _context2.sent);

          case 6:
            apps = _getApps();
            _context2.next = 9;
            return (0, _bluebird.mapSeries)(Object.keys(apps).sort(), function () {
              var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(app) {
                return _regenerator2.default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _context.next = 2;
                        return handler(app, argument);

                      case 2:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, undefined);
              }));

              return function (_x4) {
                return _ref2.apply(this, arguments);
              };
            }());

          case 9:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function executeForOneOrAll(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var _log = function _log() {
  var text = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';


  if (process.env.NODE_ENV === 'test') return;

  (0, _console.writePaddedLine)(_chalk2.default.grey(text), '', '#FFFFFF', true, false);
};

var _getRepository = function _getRepository(name) {

  if (name.match(/^https:\/\/github\.com\/.*\.git$/)) return name;

  if (name.match(/^[\w\d-]*\/[\w\d-]*$/)) return 'https://github.com/' + name + '.git';

  return 'https://github.com/mahaplatform/' + name + '.git';
};

var _getRepositoryString = function _getRepositoryString(name) {

  var full = name.match(/^https:\/\/github\.com\/([\w\d-]*)\/([\w\d-]*)\.git$/);

  if (full) return 'github:' + full[1] + '/' + full[2];

  var namespaced = name.match(/^([\w\d-]*):([\w\d-]*)\/([\w\d-]*)$/);

  if (namespaced) return name;

  return 'github:mahaplatform/' + name;
};

var _cloneRepository = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(repository) {
    var parts, name, destination;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            parts = repository.match(/\/([\w\d-]*)\.git/);

            if (parts) {
              _context3.next = 3;
              break;
            }

            throw new Error('Unable to clone ' + repository);

          case 3:
            name = parts[1];
            destination = _path2.default.join(appsPath, name);

            if (!_fs2.default.existsSync(destination)) {
              _context3.next = 7;
              break;
            }

            throw new Error(repository + ' already cloned');

          case 7:
            _context3.next = 9;
            return (0, _exec2.default)('git clone ' + repository, appsPath);

          case 9:
            return _context3.abrupt('return', name);

          case 10:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function _cloneRepository(_x6) {
    return _ref3.apply(this, arguments);
  };
}();

var _cleanApp = exports._cleanApp = function () {
  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(app) {
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:

            _log('Cleaning ' + app);

            _context4.next = 3;
            return (0, _exec2.default)('rm -rf ' + app, appsPath);

          case 3:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function _cleanApp(_x7) {
    return _ref4.apply(this, arguments);
  };
}();

var _installApp = exports._installApp = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(app) {
    var repository;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            if (!_fs2.default.existsSync(_path2.default.join(appsPath, app))) {
              _context5.next = 5;
              break;
            }

            _log('Updating ' + app);

            _context5.next = 4;
            return (0, _exec2.default)('git pull', _path2.default.join(appsPath, app));

          case 4:
            return _context5.abrupt('return', _context5.sent);

          case 5:

            _log('Installing ' + app);

            repository = _getRepository(app);
            _context5.next = 9;
            return _cloneRepository(repository);

          case 9:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  }));

  return function _installApp(_x8) {
    return _ref5.apply(this, arguments);
  };
}();

var _pullApp = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(name) {
    var appPath;
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:

            _log('Pulling app ' + name);

            appPath = _path2.default.join(appsPath, name);
            _context6.next = 4;
            return (0, _exec2.default)('git pull', appPath);

          case 4:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  }));

  return function _pullApp(_x9) {
    return _ref6.apply(this, arguments);
  };
}();

var _commitApp = function () {
  var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(name, message) {
    var appPath;
    return _regenerator2.default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:

            _log('Commiting app ' + name);

            appPath = _path2.default.join(appsPath, name);
            _context7.next = 4;
            return (0, _exec2.default)('git add .', appPath);

          case 4:
            _context7.next = 6;
            return (0, _exec2.default)('git commit -m \'' + message + '\'', appPath);

          case 6:
            _context7.next = 8;
            return (0, _exec2.default)('git push', appPath);

          case 8:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, undefined);
  }));

  return function _commitApp(_x10, _x11) {
    return _ref7.apply(this, arguments);
  };
}();

var _addApp = function () {
  var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(app) {
    var repository, config;
    return _regenerator2.default.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            repository = _getRepository(app);


            _log('Adding ' + repository);

            _context8.next = 4;
            return _cloneRepository(repository);

          case 4:
            config = _readConfig();


            config.apps[app] = _getRepositoryString(app);

            _writeConfig(config);

          case 7:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, undefined);
  }));

  return function _addApp(_x12) {
    return _ref8.apply(this, arguments);
  };
}();

var _removeApp = function () {
  var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(app) {
    var config;
    return _regenerator2.default.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:

            _log('Removing ' + app);

            _context9.next = 3;
            return (0, _exec2.default)('rm -rf ' + app, appsPath);

          case 3:
            config = _readConfig();


            delete config.apps[app];

            _writeConfig(config);

          case 6:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, undefined);
  }));

  return function _removeApp(_x13) {
    return _ref9.apply(this, arguments);
  };
}();

var addTask = exports.addTask = function () {
  var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(flags, args) {
    return _regenerator2.default.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            _context10.next = 2;
            return _addApp(args.app);

          case 2:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee10, undefined);
  }));

  return function addTask(_x14, _x15) {
    return _ref10.apply(this, arguments);
  };
}();

var removeTask = exports.removeTask = function () {
  var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(flags, args) {
    return _regenerator2.default.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            _context11.next = 2;
            return _removeApp(args.app);

          case 2:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee11, undefined);
  }));

  return function removeTask(_x16, _x17) {
    return _ref11.apply(this, arguments);
  };
}();

var installTask = exports.installTask = function () {
  var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(flags, args) {
    return _regenerator2.default.wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            _context12.next = 2;
            return executeForOneOrAll(_installApp, args.app);

          case 2:
          case 'end':
            return _context12.stop();
        }
      }
    }, _callee12, undefined);
  }));

  return function installTask(_x18, _x19) {
    return _ref12.apply(this, arguments);
  };
}();

var cleanTask = exports.cleanTask = function () {
  var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(flags, args) {
    return _regenerator2.default.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            _context13.next = 2;
            return executeForOneOrAll(_cleanApp, args.app);

          case 2:
          case 'end':
            return _context13.stop();
        }
      }
    }, _callee13, undefined);
  }));

  return function cleanTask(_x20, _x21) {
    return _ref13.apply(this, arguments);
  };
}();

var commitTask = exports.commitTask = function () {
  var _ref14 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14(flags, args) {
    return _regenerator2.default.wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            _context14.next = 2;
            return executeForOneOrAll(_commitApp, args.app, args.message);

          case 2:
          case 'end':
            return _context14.stop();
        }
      }
    }, _callee14, undefined);
  }));

  return function commitTask(_x22, _x23) {
    return _ref14.apply(this, arguments);
  };
}();

var pullTask = exports.pullTask = function () {
  var _ref15 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee15(flags, args) {
    return _regenerator2.default.wrap(function _callee15$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            _context15.next = 2;
            return executeForOneOrAll(_pullApp, args.app);

          case 2:
          case 'end':
            return _context15.stop();
        }
      }
    }, _callee15, undefined);
  }));

  return function pullTask(_x24, _x25) {
    return _ref15.apply(this, arguments);
  };
}();