'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.compile = exports.buildServer = exports.buildApp = undefined;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _console = require('../../utils/console');

var _webpack = require('./webpack.client');

var _webpack2 = _interopRequireDefault(_webpack);

var _babelCore = require('babel-core');

var _precompile = require('./precompile');

var _precompile2 = _interopRequireDefault(_precompile);

var _moveConcurrently = require('move-concurrently');

var _moveConcurrently2 = _interopRequireDefault(_moveConcurrently);

var _webpack3 = require('webpack');

var _webpack4 = _interopRequireDefault(_webpack3);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ejs = require('ejs');

var _ejs2 = _interopRequireDefault(_ejs);

var _ncp = require('ncp');

var _ncp2 = _interopRequireDefault(_ncp);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var apps = process.env.APPS.split(',');

var templateRoot = _path2.default.resolve(__dirname, '..', '..', '..', 'src', 'tasks', 'compile', 'templates');

var buildTemplates = function buildTemplates(root, templates, variables) {
  return templates.map(function (entity) {

    var templatePath = _path2.default.join(templateRoot, entity + '.ejs');

    var template = _fs2.default.readFileSync(templatePath, 'utf8');

    var filepath = _path2.default.join(root, entity);

    _fs2.default.writeFileSync(filepath, _ejs2.default.render(template, variables));
  });
};

var compileClient = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(_ref2) {
    var src = _ref2.src,
        dest = _ref2.dest,
        variables = _ref2.variables;
    var stats, time;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:

            (0, _console.writePaddedLine)(_chalk2.default.grey('Compiling client'));

            _mkdirp2.default.sync(_path2.default.join(dest, 'js'));

            _context.next = 4;
            return new _bluebird2.default(function (resolve, reject) {

              (0, _webpack4.default)((0, _webpack2.default)(apps, src, dest)).run(function (err, stats) {

                if (err) reject(err);

                resolve(stats);
              });
            });

          case 4:
            stats = _context.sent;
            time = (stats.endTime - stats.startTime) / 1000;


            (0, _console.writePaddedLine)(_chalk2.default.grey('Finished in ' + time));

          case 7:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function compileClient(_x) {
    return _ref.apply(this, arguments);
  };
}();

var getItemType = function getItemType(item) {
  return _path2.default.extname(item).length > 0 ? _path2.default.extname(item).substr(1) : 'dir';
};

var getItem = function getItem(src, root, item) {
  return {
    src: src,
    type: getItemType(item)
  };
};

var listContents = function listContents(src, root, item) {
  return [getItem(src, root, item)].concat((0, _toConsumableArray3.default)(_fs2.default.lstatSync(src).isDirectory() ? listItems(src) : []));
};

var listItems = function listItems(root) {
  return _fs2.default.readdirSync(root).reduce(function (items, item) {
    return [].concat((0, _toConsumableArray3.default)(items), (0, _toConsumableArray3.default)(listContents(_path2.default.join(root, item), root, item)));
  }, []);
};

var transpileFile = function transpileFile(src, dest) {

  (0, _console.action)('compile', src);

  var source = _fs2.default.readFileSync(src, 'utf8');

  var transpiled = (0, _babelCore.transform)(source, {
    presets: ['babel-preset-es2015', 'babel-preset-react', 'babel-preset-stage-0'],
    plugins: ['transform-promise-to-bluebird', ['transform-runtime', { polyfill: false }], ['module-resolver', {
      alias: apps.reduce(function (aliases, app) {
        return (0, _extends4.default)({}, aliases, (0, _defineProperty3.default)({}, app, _path2.default.resolve('dist', 'apps', app, 'src', 'server.js')));
      }, {})
    }]],
    sourceMaps: 'inline'
  });

  _fs2.default.writeFileSync(dest, transpiled.code);
};

var mkdir = function mkdir(src, dest) {

  (0, _console.action)('mkdir', src);

  _mkdirp2.default.sync(dest);
};

var copy = function copy(src, dest) {

  (0, _console.action)('copy', src);

  return (0, _bluebird.promisify)(_ncp2.default)(src, dest);
};

var buildItem = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(item, srcPath, destPath) {
    var dest;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            dest = item.src.replace(srcPath, destPath);

            if (!(item.type === 'js')) {
              _context2.next = 3;
              break;
            }

            return _context2.abrupt('return', transpileFile(item.src, dest));

          case 3:
            if (!(item.type === 'dir')) {
              _context2.next = 5;
              break;
            }

            return _context2.abrupt('return', mkdir(item.src, dest));

          case 5:
            _context2.next = 7;
            return copy(item.src, dest);

          case 7:
            return _context2.abrupt('return', _context2.sent);

          case 8:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function buildItem(_x2, _x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

var buildItems = function () {
  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(srcPath, destPath) {
    var items;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            items = listItems(srcPath);
            _context3.next = 3;
            return (0, _bluebird.mapSeries)(items, function (item) {
              return buildItem(item, srcPath, destPath);
            });

          case 3:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function buildItems(_x5, _x6) {
    return _ref4.apply(this, arguments);
  };
}();

var buildApp = exports.buildApp = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(app) {
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return buildItems(_path2.default.resolve('apps', app, 'src'), _path2.default.join('dist.staged', 'apps', app, 'src'));

          case 2:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function buildApp(_x7) {
    return _ref5.apply(this, arguments);
  };
}();

var buildServer = exports.buildServer = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
    var templates;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return (0, _bluebird.mapSeries)(process.env.APPS.split(','), buildApp);

          case 2:
            templates = ['cron.js', 'server.js', 'socket.js', 'worker.js'];


            buildTemplates(_path2.default.resolve('dist.staged'), templates, {});

          case 4:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  }));

  return function buildServer() {
    return _ref6.apply(this, arguments);
  };
}();

var compile = exports.compile = function () {
  var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(flags, args) {
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:

            _rimraf2.default.sync(_path2.default.resolve('dist.staged'));

            _context6.next = 3;
            return compileClient({
              src: _path2.default.resolve('apps', 'maha', 'src', 'admin'),
              dest: _path2.default.resolve('dist.staged', 'public', 'admin'),
              variables: (0, _precompile2.default)()
            });

          case 3:
            _context6.next = 5;
            return buildServer();

          case 5:

            _rimraf2.default.sync(_path2.default.resolve('dist'));

            _context6.next = 8;
            return (0, _moveConcurrently2.default)(_path2.default.resolve('dist.staged'), _path2.default.resolve('dist'));

          case 8:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  }));

  return function compile(_x8, _x9) {
    return _ref7.apply(this, arguments);
  };
}();