'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _htmlWebpackExcludeAssetsPlugin = require('html-webpack-exclude-assets-plugin');

var _htmlWebpackExcludeAssetsPlugin2 = _interopRequireDefault(_htmlWebpackExcludeAssetsPlugin);

var _maha_webpack_plugin = require('../../utils/maha_webpack_plugin');

var _maha_webpack_plugin2 = _interopRequireDefault(_maha_webpack_plugin);

var _miniCssExtractPlugin = require('mini-css-extract-plugin');

var _miniCssExtractPlugin2 = _interopRequireDefault(_miniCssExtractPlugin);

var _terserWebpackPlugin = require('terser-webpack-plugin');

var _terserWebpackPlugin2 = _interopRequireDefault(_terserWebpackPlugin);

var _htmlWebpackPlugin = require('html-webpack-plugin');

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _copyWebpackPlugin = require('copy-webpack-plugin');

var _copyWebpackPlugin2 = _interopRequireDefault(_copyWebpackPlugin);

var _utils = require('./utils');

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _cssnano = require('cssnano');

var _cssnano2 = _interopRequireDefault(_cssnano);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var webpackConfig = function webpackConfig(apps, src, dest) {
  return {
    devtool: 'none',
    entry: {
      bundle: _path2.default.resolve('tmp', 'index.js'),
      style: _path2.default.resolve('tmp', 'index.less')
    },
    mode: 'production',
    module: {
      rules: [{
        test: (0, _utils.ruleTest)(/\.less$/),
        use: [_miniCssExtractPlugin2.default.loader, { loader: 'css-loader', options: {
            url: false, sourceMap: false }
        }, { loader: 'postcss-loader', options: {
            plugins: [_autoprefixer2.default, _cssnano2.default] }
        }, 'less-loader']
      }, {
        test: (0, _utils.ruleTest)(/\.js$/),
        loader: 'babel-loader',
        options: {
          cacheDirectory: true,
          presets: ['es2015', 'react', 'stage-0']
        }
      }]
    },
    optimization: {
      minimizer: [new _terserWebpackPlugin2.default({
        terserOptions: {
          mangle: true,
          safari10: true
        }
      })],
      splitChunks: {
        cacheGroups: {
          commons: {
            test: _utils.chunkTest,
            name: 'vendors',
            chunks: 'all'
          }
        }
      }
    },
    output: {
      path: dest,
      filename: 'js/[name]-[chunkhash].min.js',
      publicPath: '/admin'
    },
    plugins: [new _maha_webpack_plugin2.default(), new _miniCssExtractPlugin2.default({
      filename: 'css/[name]-[hash].min.css'
    }), new _copyWebpackPlugin2.default([{ from: _path2.default.join(src, 'public'), to: dest }]), new _htmlWebpackPlugin2.default({
      template: _path2.default.join(src, 'index.html'),
      excludeAssets: [/style.*js$/]
    }), new _htmlWebpackExcludeAssetsPlugin2.default(), new _webpack2.default.SourceMapDevToolPlugin({
      filename: '[file].map',
      exclude: [/(vendors|style)/]
    }), new _webpack2.default.DefinePlugin({
      'process.env': {
        'DOMAIN': JSON.stringify(process.env.DOMAIN || 'localhost'),
        'DEVSERVER_PORT': JSON.stringify(process.env.DEVSERVER_PORT),
        'SERVER_PORT': JSON.stringify(process.env.SERVER_PORT),
        'SOCKET_PORT': JSON.stringify(process.env.SOCKET_PORT),
        'FCM_API_KEY': JSON.stringify(process.env.FCM_API_KEY),
        'FCM_MESSAGE_SENDER_ID': JSON.stringify(process.env.FCM_MESSAGE_SENDER_ID),
        'VAPID_PUBLIC_KEY': JSON.stringify(process.env.VAPID_PUBLIC_KEY),
        'GOOGLE_TRACKING_ID': JSON.stringify(process.env.GOOGLE_TRACKING_ID || ''),
        'WEB_HOST': JSON.stringify(process.env.WEB_HOST),
        'WEB_ASSET_CDN_HOST': JSON.stringify(process.env.WEB_ASSET_CDN_HOST),
        'DATA_ASSET_CDN_HOST': JSON.stringify(process.env.DATA_ASSET_CDN_HOST),
        'WEB_ASSET_HOST': JSON.stringify(process.env.WEB_ASSET_HOST),
        'DATA_ASSET_HOST': JSON.stringify(process.env.DATA_ASSET_HOST),
        'ROLLBAR_CLIENT_TOKEN': JSON.stringify(process.env.ROLLBAR_CLIENT_TOKEN)
      }
    })],
    resolve: {
      alias: (0, _extends4.default)({}, apps.reduce(function (aliases, app) {
        return (0, _extends4.default)({}, aliases, (0, _defineProperty3.default)({}, app, _path2.default.resolve('apps', app, 'src', 'client.js')));
      }, {}), {
        'maha-admin': _path2.default.resolve('apps', 'maha', 'src', 'client.js'),
        'maha-client': _path2.default.resolve('apps', 'maha', 'src', 'admin', 'index.js')
      })
    },
    resolveLoader: {
      modules: [_path2.default.resolve(__dirname, '..', '..', '..', 'node_modules')]
    }
  };
};

exports.default = webpackConfig;