'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deploy = undefined;

var _bluebird = require('bluebird');

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _sshPool = require('ssh-pool');

var _sshPool2 = _interopRequireDefault(_sshPool);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var values = require(_path2.default.resolve('config', 'deploy.js'));

var config = (0, _extends4.default)({}, values.default, values.production);

var timestamp = (0, _moment2.default)().format('YYYYMMDDHHmmss');

var sharedPath = _path2.default.join(config.deployTo, 'shared');

var currentPath = _path2.default.join(config.deployTo, 'current');

var releasePath = _path2.default.join(config.deployTo, 'releases', timestamp);

var servers = config.servers.reduce(function (servers, server) {

  var fullname = server.user + '@' + server.host;

  return (0, _extends4.default)({}, servers, {
    all: [].concat((0, _toConsumableArray3.default)(servers.all || []), (0, _toConsumableArray3.default)(_lodash2.default.includes(servers.all, fullname) ? [] : [fullname]))
  }, _lodash2.default.castArray(server.role).reduce(function (roles, role) {
    return (0, _extends4.default)({}, roles, (0, _defineProperty3.default)({}, role, [].concat((0, _toConsumableArray3.default)(servers[role] || []), [fullname])));
  }, {}));
}, {});

var options = {
  stdout: process.stdout,
  stderr: process.stderr,
  log: console.log.bind(console),
  key: config.key
};

var getPool = function getPool(roles) {

  if (!roles) return new _sshPool2.default.Connection((0, _extends4.default)({
    remote: 'root@localhost'
  }, options));

  var hosts = _lodash2.default.castArray(roles).reduce(function (hosts, role) {
    return [].concat((0, _toConsumableArray3.default)(hosts), (0, _toConsumableArray3.default)(servers[role].filter(function (host) {
      return !_lodash2.default.includes(hosts, host);
    })));
  }, []);

  return new _sshPool2.default.ConnectionPool(hosts, options);
};

var local = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(connection, cmd) {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return connection.runLocally(cmd);

          case 2:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function local(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var run = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(pool, cmd) {
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return pool.run(cmd);

          case 2:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function run(_x3, _x4) {
    return _ref2.apply(this, arguments);
  };
}();

var sync = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(pool, paths) {
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return pool.copyToRemote(paths[0], paths[1], {
              rsync: ['--delete']
            });

          case 2:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function sync(_x5, _x6) {
    return _ref3.apply(this, arguments);
  };
}();

var getAction = function getAction(action) {

  if (action === 'run') return run;

  if (action === 'sync') return sync;

  if (action === 'local') return local;
};

var getDuration = function getDuration(startTime) {

  var diff = process.hrtime(startTime);

  var ms = (diff[0] * 1e3 + diff[1] * 1e-6) / 1000;

  return ms.toFixed(3) + 's';
};

var getFunction = function getFunction(options, direction) {

  if (!options[direction]) return function () {};

  var fn = getAction(options.action);

  var pool = getPool(options.roles);

  return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
    var startTime, action;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            startTime = process.hrtime();
            action = direction === 'up' ? 'DEPLOY' : 'ROLLBACK';


            process.stdout.write(_chalk2.default.green('\n[' + action + ']') + _chalk2.default.white(': ' + options.description + '\n'));

            _context4.next = 5;
            return fn(pool, options[direction]);

          case 5:

            process.stdout.write(_chalk2.default.red('Completed in ' + getDuration(startTime) + '\n'));

          case 6:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));
};

var task = function task(options) {
  return {
    up: getFunction(options, 'up'),
    down: getFunction(options, 'down')
  };
};

var makeReleasePath = task({
  description: 'Making release path',
  action: 'run',
  up: 'rm -rf ' + releasePath + ' && mkdir -p ' + releasePath,
  down: 'rm -rf ' + releasePath,
  roles: ['appserver', 'worker', 'cron']
});

var copyPackage = task({
  description: 'Copying package files',
  action: 'sync',
  up: [_path2.default.join('package*json'), releasePath],
  roles: ['appserver', 'worker', 'cron']
});

var copyNodeModules = task({
  description: 'Copying previous node_modules',
  action: 'run',
  up: 'cp -rp ' + currentPath + '/node_modules ' + releasePath + '/node_modules',
  roles: ['appserver', 'worker', 'cron']
});

var installNewModules = task({
  description: 'Installing new node_modules',
  action: 'run',
  up: 'cd ' + currentPath + ' && npm install',
  roles: ['appserver', 'worker', 'cron']
});

var copyConfig = task({
  description: 'Syncing config',
  action: 'sync',
  up: [_path2.default.join('config', 'maha.production.js'), _path2.default.join(releasePath, 'maha.js')],
  roles: ['appserver', 'worker', 'cron']
});

var compile = task({
  description: 'Compile source',
  action: 'local',
  up: 'NODE_ENV=production maha compile'
});

var syncCompiled = task({
  description: 'Syncing newly compiled src',
  action: 'sync',
  up: [_path2.default.join('dist'), releasePath],
  roles: ['appserver', 'worker', 'cron']
});

var buildHelp = task({
  description: 'Build help files',
  action: 'local',
  up: 'NODE_ENV=production maha help:build'
});

var syncHelp = task({
  description: 'Syncing newly compiled help',
  action: 'sync',
  up: [_path2.default.join('help'), releasePath],
  roles: ['appserver']
});

var linkImagecache = task({
  description: 'Linking imagecahe directory',
  action: 'run',
  up: 'ln -s ' + sharedPath + '/imagecache ' + releasePath + '/dist/public/imagecache',
  roles: 'appserver'
});

var linkLogs = task({
  description: 'Linking log directory',
  action: 'run',
  up: 'ln -s ' + sharedPath + '/logs ' + releasePath + '/logs',
  roles: ['appserver', 'worker', 'cron']
});

var symlinkRelease = task({
  description: 'Symlnk release',
  action: 'run',
  up: 'rm -rf ' + currentPath + ' && ln -s ' + currentPath + ' ' + releasePath,
  roles: ['appserver', 'worker', 'cron']
});

var bootstrap = task({
  description: 'Bootstrapping maha',
  action: 'run',
  up: 'cd ' + currentPath + ' && NODE_ENV=production maha bootstrap',
  roles: 'controller'
});

var restartLoadBalancer = task({
  description: 'Restart load balancer',
  action: 'run',
  up: 'service haproxy restart',
  roles: 'loadbalancer'
});

var reloadAppServers = task({
  description: 'Reload appservers',
  action: 'run',
  up: 'touch ' + currentPath + '/tmp/restart.txt',
  roles: 'appserver'
});

var restartAppServers = task({
  description: 'Reload appservers',
  action: 'run',
  up: 'service nginx restart',
  roles: 'appserver'
});

var cacheAppservers = task({
  description: 'Cache servers',
  action: 'run',
  up: 'wget -O - http://127.0.0.1:80/ping && wget -O - http://127.0.0.1:81/ping',
  roles: 'appserver'
});

var restartWorkers = task({
  description: 'Restart workers',
  action: 'run',
  up: 'cd ' + currentPath + ' && NODE_ENV=production pm2 startOrRestart ./config/ecosystem.config.js',
  roles: 'worker'
});

var restartCron = task({
  description: 'Restart cron',
  action: 'run',
  up: 'cd ' + currentPath + ' && NODE_ENV=production pm2 startOrRestart ./config/ecosystem.config.js',
  roles: 'cron'
});

var deploy = exports.deploy = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(flags, args) {
    var tasks, completed, startTime;
    return _regenerator2.default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            tasks = [makeReleasePath, copyPackage, copyNodeModules, installNewModules, copyConfig, compile, syncCompiled, buildHelp, syncHelp, linkImagecache, linkLogs, symlinkRelease, bootstrap, reloadAppServers, cacheAppservers, restartWorkers, restartCron];
            completed = [];
            startTime = process.hrtime();
            _context7.prev = 3;
            _context7.next = 6;
            return (0, _bluebird.mapSeries)(tasks, function () {
              var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(task) {
                return _regenerator2.default.wrap(function _callee5$(_context5) {
                  while (1) {
                    switch (_context5.prev = _context5.next) {
                      case 0:
                        _context5.next = 2;
                        return task.up();

                      case 2:

                        completed.unshift(task);

                      case 3:
                      case 'end':
                        return _context5.stop();
                    }
                  }
                }, _callee5, undefined);
              }));

              return function (_x9) {
                return _ref6.apply(this, arguments);
              };
            }());

          case 6:
            _context7.next = 13;
            break;

          case 8:
            _context7.prev = 8;
            _context7.t0 = _context7['catch'](3);


            process.stderr.write(_context7.t0);

            _context7.next = 13;
            return (0, _bluebird.mapSeries)(completed, function () {
              var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(task) {
                return _regenerator2.default.wrap(function _callee6$(_context6) {
                  while (1) {
                    switch (_context6.prev = _context6.next) {
                      case 0:
                        _context6.next = 2;
                        return task.down();

                      case 2:
                      case 'end':
                        return _context6.stop();
                    }
                  }
                }, _callee6, undefined);
              }));

              return function (_x10) {
                return _ref7.apply(this, arguments);
              };
            }());

          case 13:

            process.stdout.write(_chalk2.default.green('\nCompleted in total ' + getDuration(startTime) + '\n'));

          case 14:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, undefined, [[3, 8]]);
  }));

  return function deploy(_x7, _x8) {
    return _ref5.apply(this, arguments);
  };
}();