'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Destroy = exports.Generate = undefined;

var _bluebird = require('bluebird');

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _console = require('../../utils/console');

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

var _exec = require('../../utils/exec');

var _exec2 = _interopRequireDefault(_exec);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _ejs = require('ejs');

var _ejs2 = _interopRequireDefault(_ejs);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var modulesPath = _path2.default.resolve('node_modules');

var appsPath = _path2.default.resolve('app');

var generatorPath = _path2.default.resolve(__dirname, '..', '..', '..', 'src', 'generators');

var generators = _fs2.default.readdirSync(generatorPath);

var Generate = exports.Generate = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(flags, args) {
    var template, app, name, generatorPath, templatesPath, appConfigFilePath, appConfig, data, generator;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            template = args.template;
            app = args.app;
            name = typeof args.name === 'string' ? args.name : args.app;
            generatorPath = generators.find(function (generator) {
              return generator == template;
            });

            if (generatorPath) {
              _context.next = 6;
              break;
            }

            throw new Error('invalid template');

          case 6:
            templatesPath = _path2.default.join(generatorPath, template);
            appConfigFilePath = _path2.default.join(_getAppRoot(app), app, 'src', 'app.js');
            appConfig = _fs2.default.existsSync(appConfigFilePath) ? require(appConfigFilePath).default.config : null;
            data = getData(template, app, appConfig, name);
            generator = require(generatorFilePath).default;


            generator.files.map(function (file) {
              return generate(file, templatesPath, data);
            });

            if (!generator.after) {
              _context.next = 15;
              break;
            }

            _context.next = 15;
            return runHooks(generator.after, data);

          case 15:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function Generate(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var Destroy = exports.Destroy = function Destroy(template, app, name) {

  var generatorFilePath = _path2.default.join(modulesPath, 'maha', 'dist', 'generators', template, 'index.js');

  if (!_fs2.default.existsSync(generatorFilePath)) return;

  var appConfigFilePath = _path2.default.join(modulesPath, app, 'dist', 'app.js');

  var appConfig = _fs2.default.existsSync(appConfigFilePath) ? require(appConfigFilePath).default.config : null;

  var data = getData(template, appConfig, name);

  var generator = require(generatorFilePath).default;

  generator.files.map(function (file) {
    return destroy(file, data);
  });
};

var generate = function generate(options, templatesPath, data) {

  var appPath = _ejs2.default.render(data.app + '/' + options.filepath, data);

  var fullPath = appPath.split('/').slice(0, -1).reduce(function (fullPath, segment) {

    var joinedPath = _path2.default.join(fullPath, segment);

    var directoryPath = _path2.default.join(data.root, joinedPath);

    if (!_fs2.default.existsSync(directoryPath)) {

      (0, _console.writePaddedLine)(_chalk2.default.green('mkdir') + '     ' + joinedPath);

      _fs2.default.mkdirSync(directoryPath);
    }

    return joinedPath;
  }, '');

  var rendered = getRendered(options, templatesPath, data);

  var filename = _ejs2.default.render(options.filepath.split('/').slice(-1)[0], data);

  var filePath = _path2.default.join(fullPath, filename);

  var exists = _fs2.default.readdirSync(_path2.default.join(data.root, fullPath)).reduce(function (exists, file) {

    return exists || file.replace(data.timestamp, '').search(filename.replace(data.timestamp, '')) >= 0;
  }, false);

  if (options.action === 'create') {

    if (exists) return (0, _console.writePaddedLine)(_chalk2.default.hex('#2185D0')('identical') + ' ' + filePath);

    (0, _console.writePaddedLine)(_chalk2.default.green('write') + '     ' + filePath);

    _fs2.default.writeFileSync(_path2.default.join(data.root, filePath), rendered);
  } else if (options.action === 'prepend') {

    if (!exists) return (0, _console.writePaddedLine)(_chalk2.default.hex('#DB2828')('notfound') + '    ' + filePath);

    var contents = _fs2.default.readFileSync(_path2.default.resolve(filePath)).toString();

    var prepended = options.before ? contents.replace(options.before, '' + rendered + options.before) : rendered + '\n' + contents;

    _fs2.default.writeFileSync(_path2.default.join(data.root, filePath), prepended);

    (0, _console.writePaddedLine)(_chalk2.default.green('alter') + '    ' + filePath);
  } else if (options.action === 'append') {

    if (!exists) return (0, _console.writePaddedLine)(_chalk2.default.hex('#DB2828')('notfound') + '    ' + filePath + '\n');

    var _contents = _fs2.default.readFileSync(_path2.default.resolve(filePath)).toString();

    var appended = options.after ? _contents.replace(options.after, '' + options.after + rendered) : _contents + '\n' + rendered;

    _fs2.default.writeFileSync(_path2.default.join(data.root, filePath), appended);

    (0, _console.writePaddedLine)(_chalk2.default.green('alter') + '    ' + filePath);
  }
};

var runHooks = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(hooks, data) {
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return (0, _bluebird.mapSeries)(hooks, function () {
              var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(hook) {
                return _regenerator2.default.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:

                        (0, _console.writePaddedLine)(_chalk2.default.green('exec') + '      ' + hook.description);

                        _context2.next = 3;
                        return (0, _exec2.default)(hook.command, data.root + '/' + data.path);

                      case 3:
                      case 'end':
                        return _context2.stop();
                    }
                  }
                }, _callee2, undefined);
              }));

              return function (_x5) {
                return _ref3.apply(this, arguments);
              };
            }());

          case 2:
            return _context3.abrupt('return', _context3.sent);

          case 3:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function runHooks(_x3, _x4) {
    return _ref2.apply(this, arguments);
  };
}();

var destroy = function destroy(options, data) {

  var fullPath = options.filepath.split('/').slice(0, -1).reduce(function (fullPath, segment) {

    var interpolated = _ejs2.default.render(segment, data);

    return _path2.default.join(fullPath, interpolated);
  }, '');

  var filename = _ejs2.default.render(options.filepath.split('/').slice(-1)[0], data);

  var filePath = _path2.default.join(fullPath, filename);

  if (!_fs2.default.existsSync(_path2.default.resolve(filePath))) return;

  (0, _console.writePaddedLine)(_chalk2.default.green('delete') + '     ' + filePath);

  _fs2.default.unlinkSync(_path2.default.resolve(filePath));
};

var getRendered = function getRendered(options, templatesPath, data) {

  if (options.template) {

    var templateFilePath = _path2.default.join(templatesPath, options.template);

    var template = _fs2.default.readFileSync(templateFilePath, 'utf8');

    return _ejs2.default.render(template, data);
  } else if (options.content) {

    return _ejs2.default.render(options.content, data);
  } else {

    return '';
  }
};

var getData = function getData(template, app, appConfig, initname) {

  var parts = initname.split('/').map(_lodash2.default.snakeCase);

  var name = parts[parts.length - 1];

  var app_name = _lodash2.default.includes(['platform', 'app'], template) ? _lodash2.default.kebabCase(name) : appConfig.title;

  return {
    _: _lodash2.default,
    app: app,
    name: name,
    app_name: app_name,
    root: template == 'app' ? appsPath : _getAppRoot(app),
    path: parts.join('/'),
    pluralize: _pluralize2.default,
    timestamp: (0, _moment2.default)().format('YYYYMMDDHHmmss')
  };
};

var _getAppRoot = function _getAppRoot(app) {

  return _fs2.default.existsSync(_path2.default.join(modulesPath, app)) ? modulesPath : appsPath;
};