'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.build = exports.buildApp = undefined;

var _bluebird = require('bluebird');

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _transpile = require('../../utils/transpile');

var _transpile2 = _interopRequireDefault(_transpile);

var _console = require('../../utils/console');

var _moveConcurrently = require('move-concurrently');

var _moveConcurrently2 = _interopRequireDefault(_moveConcurrently);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ncp = require('ncp');

var _ncp2 = _interopRequireDefault(_ncp);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getItemType = function getItemType(item) {
  return _path2.default.extname(item).length > 0 ? _path2.default.extname(item).substr(1) : 'dir';
};

var getItem = function getItem(src, root, item) {
  return {
    src: src,
    type: getItemType(item)
  };
};

var listContents = function listContents(src, root, item) {
  return [getItem(src, root, item)].concat((0, _toConsumableArray3.default)(_fs2.default.lstatSync(src).isDirectory() ? listItems(src) : []));
};

var listItems = function listItems(root) {
  return _fs2.default.readdirSync(root).reduce(function (items, item) {
    return [].concat((0, _toConsumableArray3.default)(items), (0, _toConsumableArray3.default)(listContents(_path2.default.join(root, item), root, item)));
  }, []);
};

var transpileFile = function transpileFile(src, dest) {

  (0, _console.action)('compile', src);

  var contents = _fs2.default.readFileSync(src, 'utf8');

  var transpiled = (0, _transpile2.default)(contents);

  _fs2.default.writeFileSync(dest, transpiled);
};

var mkdir = function mkdir(src, dest) {

  (0, _console.action)('mkdir', src);

  _mkdirp2.default.sync(dest);
};

var copy = function copy(src, dest) {

  (0, _console.action)('copy', src);

  return (0, _bluebird.promisify)(_ncp2.default)(src, dest);
};

var buildItem = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(item, srcPath, destPath) {
    var dest;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            dest = item.src.replace(srcPath, destPath);

            if (!(item.type === 'js')) {
              _context.next = 3;
              break;
            }

            return _context.abrupt('return', transpileFile(item.src, dest));

          case 3:
            if (!(item.type === 'dir')) {
              _context.next = 5;
              break;
            }

            return _context.abrupt('return', mkdir(item.src, dest));

          case 5:
            _context.next = 7;
            return copy(item.src, dest);

          case 7:
            return _context.abrupt('return', _context.sent);

          case 8:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function buildItem(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var buildItems = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(srcPath, destPath) {
    var items;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            items = listItems(srcPath);
            _context2.next = 3;
            return (0, _bluebird.mapSeries)(items, function (item) {
              return buildItem(item, srcPath, destPath);
            });

          case 3:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function buildItems(_x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var remove = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(dir) {
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:

            if (_fs2.default.existsSync(dir)) _rimraf2.default.sync(dir);

          case 1:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function remove(_x6) {
    return _ref3.apply(this, arguments);
  };
}();

var buildApp = exports.buildApp = function () {
  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(appRoot) {
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return remove(_path2.default.join(appRoot, 'dist.staged'));

          case 2:

            _mkdirp2.default.sync(_path2.default.join(appRoot, 'dist.staged'));

            _context4.next = 5;
            return buildItems(_path2.default.join(appRoot, 'src'), _path2.default.join(appRoot, 'dist.staged'));

          case 5:
            _context4.next = 7;
            return remove(_path2.default.join(appRoot, 'dist'));

          case 7:
            _context4.next = 9;
            return (0, _moveConcurrently2.default)(_path2.default.join(appRoot, 'dist.staged'), _path2.default.join(appRoot, 'dist'));

          case 9:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function buildApp(_x7) {
    return _ref4.apply(this, arguments);
  };
}();

var build = exports.build = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(flags, args) {
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            if (!args.app) {
              _context6.next = 4;
              break;
            }

            _context6.next = 3;
            return buildApp(_path2.default.resolve(args.app));

          case 3:
            return _context6.abrupt('return', _context6.sent);

          case 4:
            _context6.next = 6;
            return (0, _bluebird.mapSeries)(process.env.APPS.split(','), function () {
              var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(app) {
                return _regenerator2.default.wrap(function _callee5$(_context5) {
                  while (1) {
                    switch (_context5.prev = _context5.next) {
                      case 0:
                        _context5.next = 2;
                        return buildApp(_path2.default.resolve(app));

                      case 2:
                        return _context5.abrupt('return', _context5.sent);

                      case 3:
                      case 'end':
                        return _context5.stop();
                    }
                  }
                }, _callee5, undefined);
              }));

              return function (_x10) {
                return _ref6.apply(this, arguments);
              };
            }());

          case 6:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  }));

  return function build(_x8, _x9) {
    return _ref5.apply(this, arguments);
  };
}();

build();