'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pullTask = exports.cleanTask = exports.commitTask = exports.linkTask = exports.buildTask = exports.installTask = exports.importTask = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _bluebird = require('bluebird');

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _console = require('../../utils/console');

var _build = require('./build');

var _exec = require('../../utils/exec');

var _exec2 = _interopRequireDefault(_exec);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var platformPath = _path2.default.resolve('.');

var packagesPath = _path2.default.join(platformPath, 'packages');

var modulesPath = _path2.default.join(platformPath, 'node_modules');

var cachedModules = {};

var _getModules = function _getModules(modulesPath) {

  if (cachedModules[modulesPath]) return cachedModules[modulesPath];

  cachedModules[modulesPath] = _fs2.default.readdirSync(modulesPath).reduce(function (modules, moduleName) {

    var modulePath = _path2.default.join(modulesPath, moduleName);

    if (!_fs2.default.statSync(modulePath).isDirectory()) return modules;

    return (0, _extends4.default)({}, modules, (0, _defineProperty3.default)({}, moduleName, modulePath));
  }, {});

  return cachedModules[modulesPath];
};

var _validatePackage = function _validatePackage(name) {

  var packages = Object.keys(_getModules('packages'));

  if (!_lodash2.default.includes(packages, name)) throw new Error('Package ' + name + ' is not installed');
};

var executeForOneOrAll = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(handler, name, arg) {
    var argument, packages;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            argument = typeof arg === 'string' ? arg : null;

            if (!(typeof name === 'string' && name !== 'all')) {
              _context.next = 6;
              break;
            }

            _validatePackage(name);

            _context.next = 5;
            return handler(name, argument);

          case 5:
            return _context.abrupt('return', _context.sent);

          case 6:
            packages = Object.keys(_getModules(packagesPath)).sort();
            _context.next = 9;
            return (0, _bluebird.map)(packages, function (name) {
              return handler(name, argument);
            });

          case 9:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function executeForOneOrAll(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var _log = function _log() {
  var text = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';


  if (process.env.NODE_ENV === 'test') return;

  (0, _console.writePaddedLine)(_chalk2.default.grey(text), '', '#FFFFFF', true, false);
};

var _getRepository = function _getRepository(name) {

  if (name.match(/^https:\/\/github\.com\/.*\.git$/)) return name;

  if (name.match(/^[\w\d-]*\/[\w\d-]*$/)) return 'https://github.com/' + name + '.git';

  return 'https://github.com/mahaplatform/' + name + '.git';
};

var _getDependencies = function _getDependencies(packagePath) {

  var configPath = _path2.default.join(packagePath, 'package.json');

  if (!_fs2.default.existsSync(configPath)) return [];

  var config = JSON.parse(_fs2.default.readFileSync(configPath).toString());

  return [].concat((0, _toConsumableArray3.default)(config.devDependencies ? Object.keys(config.devDependencies) : []), (0, _toConsumableArray3.default)(config.dependencies ? Object.keys(config.dependencies) : []));
};

var _clonePackage = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(repo) {
    var repository, parts, name, destination;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            repository = _getRepository(repo);


            _log('Cloning ' + repository);

            parts = repository.match(/\/([\w\d-]*)\.git/);

            if (parts) {
              _context2.next = 5;
              break;
            }

            throw new Error('Unable to clone ' + repository);

          case 5:
            name = parts[1];
            destination = _path2.default.join(packagesPath, name);

            if (!_fs2.default.existsSync(destination)) {
              _context2.next = 9;
              break;
            }

            throw new Error(repository + ' already cloned');

          case 9:
            _context2.next = 11;
            return (0, _exec2.default)('git clone ' + repository, packagesPath);

          case 11:
            return _context2.abrupt('return', name);

          case 12:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function _clonePackage(_x5) {
    return _ref2.apply(this, arguments);
  };
}();

var _installPackage = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(name, dependency) {
    var description, packagePath, command;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            description = dependency ? dependency : 'all modules';


            _log('Installing ' + description + ' for package ' + name);

            packagePath = _path2.default.join(packagesPath, name);
            command = dependency ? 'npm install ' + dependency : 'npm install';
            _context3.next = 6;
            return (0, _exec2.default)(command, packagePath);

          case 6:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function _installPackage(_x6, _x7) {
    return _ref3.apply(this, arguments);
  };
}();

var _cleanPackage = function () {
  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(name) {
    var packagePath;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:

            _log('Cleaning package ' + name);

            packagePath = _path2.default.join(packagesPath, name);
            _context4.next = 4;
            return (0, _exec2.default)('rm -rf dist', packagePath);

          case 4:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function _cleanPackage(_x8) {
    return _ref4.apply(this, arguments);
  };
}();

var _pullPackage = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(name) {
    var packagePath;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:

            _log('Pulling package ' + name);

            packagePath = _path2.default.join(packagesPath, name);
            _context5.next = 4;
            return (0, _exec2.default)('git pull', packagePath);

          case 4:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  }));

  return function _pullPackage(_x9) {
    return _ref5.apply(this, arguments);
  };
}();

var _buildPackage = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(name) {
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:

            _log('Building package ' + name);

            _context6.next = 3;
            return (0, _build.build)({}, { root: 'packages', app: name });

          case 3:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  }));

  return function _buildPackage(_x10) {
    return _ref6.apply(this, arguments);
  };
}();

var _linkPackage = function () {
  var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(name) {
    var packagePath, modulePath, packages;
    return _regenerator2.default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:

            _log('Linking package ' + name);

            packagePath = _path2.default.join(packagesPath, name);
            modulePath = _path2.default.join(modulesPath, name);


            if (modulePath) {

              _rimraf2.default.sync(modulePath);

              _fs2.default.symlinkSync(packagePath, modulePath);
            }

            packages = _getModules(packagesPath);


            _getDependencies(packagePath).map(function (dependency) {

              if (!packages[dependency]) return;

              var packageModulePath = _path2.default.join(packagePath, 'node_modules', dependency);

              if (_fs2.default.statSync(packageModulePath).isDirectory()) _rimraf2.default.sync(packageModulePath);

              _fs2.default.symlinkSync(packages[dependency], packageModulePath);
            });

            _fs2.default.readdirSync(modulesPath).map(function (moduleName) {

              if (!packages[moduleName]) return;

              var modulePath = _path2.default.join(modulesPath, moduleName);

              if (_fs2.default.statSync(modulePath).isDirectory()) _rimraf2.default.sync(modulePath);

              _fs2.default.symlinkSync(packages[moduleName], modulePath);
            });

          case 7:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, undefined);
  }));

  return function _linkPackage(_x11) {
    return _ref7.apply(this, arguments);
  };
}();

var _commitPackage = function () {
  var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(name, message) {
    var packagePath;
    return _regenerator2.default.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:

            _log('Commiting package ' + name);

            packagePath = _path2.default.join(packagesPath, name);
            _context8.next = 4;
            return (0, _exec2.default)('git add .', packagePath);

          case 4:
            _context8.next = 6;
            return (0, _exec2.default)('git commit -m \'' + message + '\'', packagePath);

          case 6:
            _context8.next = 8;
            return (0, _exec2.default)('git push', packagePath);

          case 8:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, undefined);
  }));

  return function _commitPackage(_x12, _x13) {
    return _ref8.apply(this, arguments);
  };
}();

var importTask = exports.importTask = function () {
  var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(flags, args) {
    var name;
    return _regenerator2.default.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return _clonePackage(args.repository);

          case 2:
            name = _context9.sent;
            _context9.next = 5;
            return _installPackage(name);

          case 5:
            _context9.next = 7;
            return _buildPackage(name);

          case 7:
            _context9.next = 9;
            return _linkPackage(name);

          case 9:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, undefined);
  }));

  return function importTask(_x14, _x15) {
    return _ref9.apply(this, arguments);
  };
}();

var installTask = exports.installTask = function () {
  var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(name, dependency) {
    return _regenerator2.default.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            _context10.next = 2;
            return executeForOneOrAll(_installPackage, name, dependency);

          case 2:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee10, undefined);
  }));

  return function installTask(_x16, _x17) {
    return _ref10.apply(this, arguments);
  };
}();

var buildTask = exports.buildTask = function () {
  var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(name) {
    return _regenerator2.default.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            _context11.next = 2;
            return executeForOneOrAll(_buildPackage, name);

          case 2:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee11, undefined);
  }));

  return function buildTask(_x18) {
    return _ref11.apply(this, arguments);
  };
}();

var linkTask = exports.linkTask = function () {
  var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(name) {
    return _regenerator2.default.wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            _context12.next = 2;
            return executeForOneOrAll(_linkPackage, name);

          case 2:
          case 'end':
            return _context12.stop();
        }
      }
    }, _callee12, undefined);
  }));

  return function linkTask(_x19) {
    return _ref12.apply(this, arguments);
  };
}();

var commitTask = exports.commitTask = function () {
  var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(name, message) {
    return _regenerator2.default.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            _context13.next = 2;
            return executeForOneOrAll(_commitPackage, name, message);

          case 2:
          case 'end':
            return _context13.stop();
        }
      }
    }, _callee13, undefined);
  }));

  return function commitTask(_x20, _x21) {
    return _ref13.apply(this, arguments);
  };
}();

var cleanTask = exports.cleanTask = function () {
  var _ref14 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14(name, message) {
    return _regenerator2.default.wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            _context14.next = 2;
            return executeForOneOrAll(_cleanPackage, name, message);

          case 2:
          case 'end':
            return _context14.stop();
        }
      }
    }, _callee14, undefined);
  }));

  return function cleanTask(_x22, _x23) {
    return _ref14.apply(this, arguments);
  };
}();

var pullTask = exports.pullTask = function () {
  var _ref15 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee15(name, message) {
    return _regenerator2.default.wrap(function _callee15$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            _context15.next = 2;
            return executeForOneOrAll(_pullPackage, name, message);

          case 2:
          case 'end':
            return _context15.stop();
        }
      }
    }, _callee15, undefined);
  }));

  return function pullTask(_x24, _x25) {
    return _ref15.apply(this, arguments);
  };
}();