'use strict';

var path = require('path');
var utils = require('shipit-utils');
var deploy = require('shipit-deploy');
var roles = require('shipit-roles');

module.exports = function (shipit) {

  roles(shipit);

  deploy(shipit);

  var deployDir = shipit.config.deployTo;

  var sharedDir = deployDir + '/shared';

  var currentDir = deployDir + '/current';

  utils.registerTask(shipit, 'deploy:maha_prepare', ['deploy:config', 'deploy:install_apps', 'deploy:rebuild_modules', 'deploy:compile', 'deploy:link_shared_dirs', 'deploy:help', 'deploy:bootstrap', 'deploy:migrate']);

  utils.registerTask(shipit, 'deploy:maha_release', ['deploy:reload_appservers', 'deploy:restart_workers', 'deploy:restart_cron', 'deploy:cache_app', 'deploy:cache_socket']);

  utils.registerTask(shipit, 'deploy:maha_restart', ['deploy:restart_appservers', 'deploy:restart_loadbalancers', 'deploy:cache_app', 'deploy:cache_socket']);

  utils.registerTask(shipit, 'sync', ['sync:backup', 'sync:download', 'sync:restore', 'sync:passwords']);

  utils.registerTask(shipit, 'sync:backup', function () {
    return shipit.roles.dbserver.run('pg_dump -h localhost -U maha maha | gzip > backup.sql.gz');
  });

  utils.registerTask(shipit, 'sync:download', function () {
    return shipit.roles.dbserver.copy('backup.sql.gz', 'tmp/backup.sql.gz', { direction: 'remoteToLocal' });
  });

  utils.registerTask(shipit, 'sync:restore', function () {
    return shipit.local('dropdb maha && createdb maha && gunzip < tmp/backup.sql.gz | psql maha');
  });

  utils.registerTask(shipit, 'sync:passwords', function () {
    var password_salt = '\\$2a\\$10\\$EjngW3t55b8gCmtgs4a/WO';
    var password_hash = '\\$2a\\$10\\$EjngW3t55b8gCmtgs4a/WOR6KzZnF8hKHBDgmC69gz5SaQVNmhhGa';
    var sql = 'UPDATE maha_users SET password_salt=\'' + password_salt + '\', password_hash=\'' + password_hash + '\'';
    return shipit.local('echo "' + sql + '" | psql maha');
  });

  utils.registerTask(shipit, 'deploy:create_shared_dirs', function () {
    var imagecache = function imagecache() {
      return shipit.remote('mkdir -m 777 -p ' + sharedDir + '/imagecache', { role: 'appserver' });
    };
    var logs = function logs() {
      return shipit.remote('mkdir -p ' + sharedDir + '/logs');
    };
    var tmp = function tmp() {
      return shipit.remote('chmod 777 ' + sharedDir + '/tmp');
    };
    return imagecache().then(logs).then(tmp);
  });

  utils.registerTask(shipit, 'deploy:create_logs', function () {
    return shipit.remote('mkdir -p ' + sharedDir + '/logs');
  });

  utils.registerTask(shipit, 'deploy:rebuild_modules', function () {
    return shipit.remote('cd ' + shipit.releasePath + ' && rm -rf node_modules && npm install');
  });

  utils.registerTask(shipit, 'deploy:install_apps', function () {
    return shipit.remote('cd ' + shipit.releasePath + ' && rm -rf apps/* && NODE_ENV=' + shipit.options.environment + ' maha app:install');
  });

  utils.registerTask(shipit, 'deploy:config', function () {
    var mkdir = function mkdir() {
      return shipit.remote('mkdir -p ' + sharedDir);
    };
    var copyConfig = function copyConfig() {
      return shipit.remoteCopy(path.resolve('config', 'maha.' + shipit.options.environment + '.js'), sharedDir + '/maha.js');
    };
    var linkConfig = function linkConfig() {
      return shipit.remote('rm -rf ' + shipit.releasePath + '/maha.js && ln -s ' + sharedDir + '/maha.js ' + shipit.releasePath + '/maha.js');
    };
    return mkdir().then(copyConfig).then(linkConfig);
  });

  utils.registerTask(shipit, 'deploy:compile', function () {
    return shipit.remote('cd ' + shipit.releasePath + ' && NODE_ENV=' + shipit.options.environment + ' maha compile');
  });

  utils.registerTask(shipit, 'deploy:migrate', function () {
    return shipit.remote('cd ' + shipit.releasePath + ' && NODE_ENV=' + shipit.options.environment + ' maha db:migrate:up', { role: 'cron' });
  });

  utils.registerTask(shipit, 'deploy:help', function () {
    var build = function build() {
      return shipit.remote('cd ' + shipit.releasePath + ' && NODE_ENV=' + shipit.options.environment + ' maha help:build', { role: 'appserver' });
    };
    var chmown = function chmown() {
      return shipit.remote('chown -R nobody.nobody ' + shipit.releasePath + '/help', { role: 'appserver' });
    };
    var chmod = function chmod() {
      return shipit.remote('chmod -R 777 ' + shipit.releasePath + '/help', { role: 'appserver' });
    };
    return build().then(chmown).then(chmod);
  });

  utils.registerTask(shipit, 'deploy:bootstrap', function () {
    return shipit.remote('cd ' + shipit.releasePath + ' && NODE_ENV=' + shipit.options.environment + ' maha db:bootstrap', { role: 'cron' });
  });

  utils.registerTask(shipit, 'deploy:link_shared_dirs', function () {
    var imagecache = function imagecache() {
      return shipit.remote('ln -s ' + sharedDir + '/imagecache ' + shipit.releasePath + '/dist/public/imagecache', { role: 'appserver' });
    };
    var logs = function logs() {
      return shipit.remote('ln -s ' + sharedDir + '/logs ' + shipit.releasePath + '/logs');
    };
    return imagecache().then(logs);
  });

  utils.registerTask(shipit, 'deploy:restart_loadbalancers', function () {
    return shipit.remote('service haproxy restart', { role: 'loadbalancer' });
  });

  utils.registerTask(shipit, 'deploy:reload_appservers', function () {
    return shipit.remote('touch ' + currentDir + '/tmp/restart.txt', { role: 'appserver' });
  });

  utils.registerTask(shipit, 'deploy:restart_appservers', function () {
    return shipit.remote('service nginx restart', { role: 'appserver' });
  });

  utils.registerTask(shipit, 'deploy:cache_app', function () {
    return shipit.remote('wget -O - http://127.0.0.1:80/ping', { role: 'appserver' });
  });

  utils.registerTask(shipit, 'deploy:cache_socket', function () {
    return shipit.remote('wget -O - http://127.0.0.1:81/ping', { role: 'appserver' });
  });

  utils.registerTask(shipit, 'deploy:restart_workers', function () {
    return shipit.remote('cd ' + deployDir + ' && NODE_ENV=' + shipit.options.environment + ' pm2 startOrRestart ./current/config/ecosystem.config.js', { role: 'worker' });
  });

  utils.registerTask(shipit, 'deploy:restart_cron', function () {
    return shipit.remote('cd ' + deployDir + ' && NODE_ENV=' + shipit.options.environment + ' pm2 startOrRestart ./current/config/ecosystem.config.js', { role: 'cron' });
  });

  utils.registerTask(shipit, 'deploy:setup_cache', function () {
    return shipit.remote('mkdir -p -m 777 ' + sharedDir + '/imagecache', { role: 'appserver' });
  });

  utils.registerTask(shipit, 'deploy:update_cli', function () {
    return shipit.remote('npm install -g maha-cli');
  });

  shipit.on('updated', function () {
    return shipit.start('deploy:maha_prepare');
  });

  shipit.on('deployed', function () {
    return shipit.start('deploy:maha_release');
  });
};