'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.writePaddedLine = exports.output = exports.action = exports.log = exports.success = exports.error = exports.info = exports.write = undefined;

var _wrapAnsi = require('wrap-ansi');

var _wrapAnsi2 = _interopRequireDefault(_wrapAnsi);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var red = '#db2828';
var green = '#21ba45';
var blue = '#2185d0';
var white = '#FFFFFF';

var write = exports.write = function write(message) {
  return process.stdout.write(message + '\n');
};

var info = exports.info = function info(entity, message) {
  return log(blue, entity, message);
};

var error = exports.error = function error(entity, message) {
  return log(red, entity, message);
};

var success = exports.success = function success(entity, message) {
  return log(green, entity, message);
};

var log = exports.log = function log(color, entity, message) {
  return _lodash2.default.castArray(message).map(function (message) {
    return output([{ color: color, content: '[' + entity + ']' }, { color: white, content: ': ' + message }]);
  });
};

var action = exports.action = function action(_action, target) {
  return output([{ color: green, length: 10, content: _action }, { color: white, content: target }]);
};

var output = exports.output = function output(items) {

  if (_lodash2.default.isString(items)) return process.stdout.write(items);

  _lodash2.default.castArray(items).map(function (item) {

    var content = item.length ? _lodash2.default.padEnd(item.content, item.length) : item.content;

    return process.stdout.write(_chalk2.default.hex(item.color)(content));
  });

  process.stdout.write('\n');
};

var writePaddedLine = exports.writePaddedLine = function writePaddedLine(label) {
  var text = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';


  var width = process.stdout.columns;

  var labelWidth = label ? stripAnsi(label).length : 0;

  var contentWidth = width - labelWidth - 6;

  var padded = (0, _wrapAnsi2.default)(text, contentWidth, { hard: true }).split('\n').map(function (chunkLine, index) {

    var intro = label ? index === 0 ? label + ' ' : Array(labelWidth + 2).join(' ') : '';

    var line = intro + chunkLine;

    var stripped = stripAnsi(line);

    var extraWidth = width - stripped.length - 4;

    var extra = extraWidth > 0 ? Array(extraWidth).join(' ') : '';

    return _chalk2.default.grey('  ' + line + extra + '\n');
  }).join('');

  process.stdout.write(padded);
};

var stripAnsi = function stripAnsi(text) {
  return text.replace(/[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g, '');
};