'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _transpile = require('../transpile');

var _transpile2 = _interopRequireDefault(_transpile);

var _console = require('../console');

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _ejs = require('ejs');

var _ejs2 = _interopRequireDefault(_ejs);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var apps = process.env.APPS.split(',');

var configs = apps.reduce(function (configs, app) {

  var configPath = _path2.default.resolve('apps', app, 'src', 'app.js');

  var contents = _fs2.default.readFileSync(configPath, 'utf8');

  var transpiled = (0, _transpile2.default)(contents);

  var config = eval(transpiled);

  return (0, _extends4.default)({}, configs, (0, _defineProperty3.default)({}, app, config));
}, {});

var collectObjects = function collectObjects(pattern) {
  return [].concat((0, _toConsumableArray3.default)(_glob2.default.sync('apps/*/src/' + pattern)), (0, _toConsumableArray3.default)(_glob2.default.sync('apps/*/src/' + pattern + '/index.js')));
};

var extract = function extract(pattern) {
  var regex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  return collectObjects(pattern).map(function (file) {

    var matches = regex ? file.match(regex) : file.match(/apps\/([^/]*)/);

    return (0, _extends4.default)({}, configs[matches[1]], regex ? { name: _lodash2.default.camelCase(matches[2].replace('/', ' ')) } : {}, {
      app: matches[1],
      filepath: _path2.default.resolve(file)
    });
  });
};

var templatePath = _path2.default.resolve(__dirname, '..', '..', '..', 'src', 'utils', 'maha_webpack_plugin');

var renderTemplate = function renderTemplate(templateName, variables) {

  var template = _fs2.default.readFileSync(_path2.default.join(templatePath, templateName + '.ejs'), 'utf8');

  var data = _ejs2.default.render(template, variables);

  _fs2.default.writeFileSync(_path2.default.resolve('tmp', templateName), data, 'utf8');
};

var MahaWebpackPlugin = function () {
  function MahaWebpackPlugin(root) {
    (0, _classCallCheck3.default)(this, MahaWebpackPlugin);

    this.root = root;
  }

  (0, _createClass3.default)(MahaWebpackPlugin, [{
    key: 'apply',
    value: function apply(compiler) {

      var create = function create(file) {

        if (file === '/tmp/client.js') return;
        if (file) (0, _console.info)('DEV', 'Detected change in ' + file.replace(_path2.default.resolve(), ''));
        if (file) (0, _console.info)('DEV', 'Recompiling client');
        if (!file) (0, _console.info)('DEV', 'Compiling client');

        var reducers = collectObjects('admin/**/reducer.js').map(function (file) {
          var matches = file.match(/(apps\/([^/]*)\/src\/admin\/(.*)\/(.*))\/reducer.js/);
          return (0, _extends4.default)({}, configs[matches[2]], {
            app: matches[2],
            name: _lodash2.default.camelCase(matches[4].replace('/', ' ')),
            filepath: _path2.default.resolve(matches[1])
          });
        });

        var variables = {
          badges: extract('admin/badges.js'),
          roots: extract('admin/roots.js'),
          routes: extract('admin/routes.js'),
          reducers: reducers,
          styles: [{ filepath: _path2.default.resolve('node_modules', 'reframe', 'src', 'style.less') }].concat((0, _toConsumableArray3.default)(extract('admin/**/style.less'))),
          settings: extract('admin/settings.js'),
          userTasks: extract('admin/user_tasks.js'),
          userFields: extract('admin/user_fields.js'),
          userValues: extract('admin/user_values.js')
        };

        renderTemplate('index.js', variables);

        renderTemplate('client.js', variables);

        renderTemplate('index.less', variables);
      };

      var done = function done() {
        return (0, _console.info)('DEV', 'Finished compiling client');
      };

      compiler.hooks.afterEnvironment.tap('MahaWebpackPlugin', create);

      compiler.hooks.invalid.tap('MahaWebpackPlugin', create);

      compiler.hooks.done.tap('MahaWebpackPlugin', done);
    }
  }]);
  return MahaWebpackPlugin;
}();

exports.default = MahaWebpackPlugin;