import prompt from 'prompt-promise'
import path from 'path'
import fs from 'fs'
import chalk from 'chalk'

export default async () => {

  let data = await readData()

  data = await collectData(data)

  writeFile(data)

}

const filePath = path.resolve('.', '.env')

const variables = [
  { label: 'DATABASE_URL', default: 'sqlite3://:memory' },
  { label: 'REDIS_URL', default: 'redis://localhost:6379/13' },
  { label: 'SECRET', default: 'foo' },
  { label: 'SERVER_PORT', default: '8080' },
  { label: 'WORKER_PORT', default: '8081' },
  { label: 'DOMAIN', default: 'localhost' },
  { label: 'AWS_ACCESS_KEY_ID'},
  { label: 'AWS_SECRET_ACCESS_KEY'},
  { label: 'AWS_REGION' },
  { label: 'AWS_BUCKET' },
  { label: 'WEB_HOST' },
  { label: 'WEB_ASSET_HOST' },
  { label: 'DATA_ASSET_HOST' },
  { label: 'WEB_ASSET_CDN_HOST' },
  { label: 'DATA_ASSET_CDN_HOST' }
]

const readData = () => {

  if(!fs.existsSync(filePath)) return getDefaults()

  const data = fs.readFileSync(filePath).toString().split('\n').reduce((values, line) => {

    const matches = line.match(/([A-Z_]*)=(.*)/)

    if(!matches) return values

    const [ , key, value ] = matches

    return {
      ...values,
      [key]: value
    }

  }, {})

  return Promise.resolve(data)

}

const collectData = (defaults) => {

  return Promise.reduce(variables, async (data, variable) => {

    const defaultValue = defaults[variable.label]

    const value = await prompt(`${variable.label} [${chalk.grey(defaultValue)}]: `)

    return ({
      ...data,
      [`${variable.label}`]: value || defaultValue
    })

  }, {})

}

const writeFile = (data) => {

  const fileData = Object.keys(data).map(key => `${key}=${data[key]}`).join('\n')

  return fs.writeFileSync(filePath, fileData)

}

const getDefaults = () => {

  const defaults = variables.reduce((defaults, variable) => ({
    ...defaults,
    [variable.label]: variable.default || ''
  }), {})

  return defaults

}
