import transpile from '../../utils/transpile'
import { action } from '../../utils/console'
import move from 'move-concurrently'
import rimraf from 'rimraf'
import mkdirp from 'mkdirp'
import path from 'path'
import ncp from 'ncp'
import fs from 'fs'

const getItemType = (item) => path.extname(item).length > 0 ? path.extname(item).substr(1) : 'dir'

const getItem = (src, root, item) => ({
  src,
  type: getItemType(item)
})

const listContents = (src, root, item) => [
  getItem(src, root, item),
  ...fs.lstatSync(src).isDirectory() ? listItems(src) : []
]

const listItems = (root) => fs.readdirSync(root).reduce((items, item) => [
  ...items,
  ...listContents(path.join(root, item), root, item)
], [])

const transpileFile = (src, dest) => {

  action('compile', src)

  const contents = fs.readFileSync(src, 'utf8')

  const transpiled = transpile(contents)

  fs.writeFileSync(dest, transpiled)

}

const mkdir = (src, dest) => {

  action('mkdir', src)

  mkdirp.sync(dest)

}

const copy = (src, dest) => {

  action('copy', src)

  return Promise.promisify(ncp)(src, dest)

}

const buildItem = async (item, srcPath, destPath) => {

  const dest = item.src.replace(srcPath, destPath)

  if(item.type === 'js') return transpileFile(item.src, dest)

  if(item.type === 'dir') return mkdir(item.src, dest)

  return await copy(item.src, dest)

}

const buildItems = async (srcPath, destPath) => {

  const items = listItems(srcPath)

  await Promise.mapSeries(items, item => buildItem(item, srcPath, destPath))

}

const remove = async (dir) => {

  if(fs.existsSync(dir)) rimraf.sync(dir)

}

export const buildApp = async (appRoot) => {

  await remove(path.join(appRoot, 'dist.staged'))

  mkdirp.sync(path.join(appRoot, 'dist.staged'))

  await buildItems(path.join(appRoot, 'src'), path.join(appRoot, 'dist.staged'))

  await remove(path.join(appRoot, 'dist'))

  await move(path.join(appRoot, 'dist.staged'), path.join(appRoot, 'dist'))

}

export const build = async (flags, args) => {

  if(args.app) return await buildApp(path.resolve(args.app))

  await Promise.mapSeries(process.env.APPS.split(','), async app => {

    return await buildApp(path.resolve(app))

  })

}

build()
